package com.kingdee.eas.custom.synctask;

import com.kingdee.bos.dao.IObjectPK;
import com.kingdee.bos.metadata.entity.EntityViewInfo;
import com.kingdee.bos.metadata.entity.FilterInfo;
import com.kingdee.bos.metadata.entity.FilterItemInfo;
import com.kingdee.bos.metadata.entity.SelectorItemInfo;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.base.permission.UserInfo;
import com.kingdee.eas.basedata.person.PersonCollection;
import com.kingdee.eas.basedata.person.PersonInfo;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.synctask.util.HttpClientUtil;
import com.kingdee.eas.hr.ats.*;
import com.kingdee.eas.hr.ats.holidayLimit.generate.util.HRTimeWebUtils;

import org.apache.log4j.Logger;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import com.kingdee.bos.*;
import com.kingdee.eas.basedata.person.IPerson;
import com.kingdee.eas.basedata.person.PersonFactory;
import com.kingdee.eas.framework.CoreBaseCollection;
import com.kingdee.util.StringUtils;

import oadd.org.apache.commons.codec.digest.DigestUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

public class SyncTranForAtsFacadeControllerBean extends AbstractSyncTranForAtsFacadeControllerBean
{
    private static Logger logger =  Logger.getLogger(SyncTranForAtsFacadeControllerBean.class);

    private volatile Properties propt;  // ʹ volatile ֤ɼ
    private final Object configLock = new Object();

    private void loadProperties() throws BOSException {
        if (propt == null) {  // һμ
            synchronized (configLock) {
                if (propt == null) {  // ڶμ飨˫ؼ
                    Properties temp = new Properties();
                    String path = System.getProperty("EAS_HOME") + "/server/properties/abk/syncAtsConfig.properties";
                    try (FileInputStream fis = new FileInputStream(path)) {  // try-with-resources Զر
                        temp.load(fis);
                        propt = temp;  // ԭԸֵ
                    } catch (IOException e) {
                        String errorMsg = "ļʧ: " + path + " | " + e.getMessage();
                        logger.error(errorMsg, e);  // ʹ־¼ e.printStackTrace()
                        throw new BOSException(errorMsg);
                    }
                }
            }
        }
    }

    @Override
    protected void _syncPunchRecord(Context ctx, String beginDate, String endDate, int offset) throws BOSException {
        loadProperties();
        // 0. Уڽ
        DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate start = LocalDate.now();
        LocalDate end = LocalDate.now();
        //ʱΪ,ʹoffset
        if(StringUtils.isEmpty(beginDate) && StringUtils.isEmpty(endDate)){
    		if( offset > 0 ){
                //eDateoffset
                end = end.plusDays(offset);
    		}else if( offset < 0 ) {
                start = start.plusDays(offset);
    		}
        }else  if(StringUtils.isEmpty(beginDate) ){
            //ʼֶ,ڶǽ
            start = LocalDate.parse(beginDate, dateFormatter);
        }else  if(StringUtils.isEmpty(endDate)){
            //ʼ,ڶǽ
            end = LocalDate.parse(endDate, dateFormatter);
        }
        if (start.isAfter(end)) {
            throw new BOSException("ʼڲڽ");
        }

        // 1. ѭ
        LocalDate currentDay = start;
        while (!currentDay.isAfter(end)) {
            try {
                // ַ
                String dayStr = currentDay.format(dateFormatter);

                //  Token  URL 
                long currentTime = Instant.now().getEpochSecond();
                String secretKey = propt.getProperty("SECRET_KEY");
                String token = DigestUtils.md5Hex(secretKey + currentTime);
                // ѯ
                Map<String, String> queryParams = new HashMap<>();
                queryParams.put("timestamp", String.valueOf(currentTime));
                queryParams.put("token", token);

                // 壨Զʱ䷶Χ
                JSONObject requestBody = buildRequestParams(dayStr, dayStr); // ͬһ

                // ýӿ
                String urlPath = propt.getProperty("urlPath");
                String response = HttpClientUtil.sendPostRequest(urlPath, requestBody, queryParams);

                // Ӧ
                if (response != null) {
                    JSONObject result = JSONObject.parseObject(response);
                    if ("0".equals(result.getString("ret"))) {
                    	JSONObject data = result.getJSONObject("data");
                        JSONArray items = data.getJSONArray("items");
                        logger.error(" " + dayStr + " ͬӿ,ӿڹ¼: " + items.size()+"");
                        saveMJPunchCardData(ctx, items);
                    }
                }
            } catch (Exception e) {
                // ¼󵫼ִ
                logger.error(" " + currentDay + " ͬʧ: " + e.getMessage(), e);
            } finally {
                currentDay = currentDay.plusDays(1); // һ
            }
        }
    }


    
    
    
    private JSONObject buildRequestParams(String startDate, String endDate) {
        JSONObject params = new JSONObject();
        DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        DateTimeFormatter datetimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        
        LocalDate today = LocalDate.now();
        //  startDateĬΪ 00:00:00
        if (StringUtils.isEmpty(startDate)) {
            startDate = today.atStartOfDay().format(datetimeFormatter);
        } else {
            startDate = LocalDate.parse(startDate, dateFormatter)
                                .atStartOfDay()
                                .format(datetimeFormatter);
        }
        
        //  endDateĬΪ 23:59:59
        if (StringUtils.isEmpty(endDate)) {
            endDate = today.atTime(23, 59, 59).format(datetimeFormatter);
        } else {
            endDate = LocalDate.parse(endDate, dateFormatter)
                              .atTime(23, 59, 59)
                              .format(datetimeFormatter);
        }
        params.put("starttime", startDate);
        params.put("endtime", endDate);
        return params;
    }
    
    


    /**
     * Ž򿨼¼
     * @param ctx
     * @param jsonArray
     */
    public void saveMJPunchCardData(Context ctx, JSONArray jsonArray) {
        logger.error("saveMJPunchCardData----");
        try {
            int success = 0;
            Set<String> setCardId = new HashSet();

            CoreBaseCollection cardColl = new CoreBaseCollection();
            //ûпڵԱ

            //ȡԱ
            IPerson iPerson = PersonFactory.getLocalInstance(ctx);
            PersonCollection personCollection = iPerson.getPersonCollection("select number ");
            Set<String> persNoSet = new HashSet();
            for (int i = 0; i < personCollection.size(); i++) {
                PersonInfo personInfo = personCollection.get(i);
                persNoSet.add(personInfo.getNumber().toString());
            }

            //"id": 928547,
            //				"pin": "1234",
            //				"ename": "ϼ",
            //				"deptnumber": "1",
            //				"deptname": "ܲ",
            //				"checktime": "2025-04-25 00:00:12",
            //				"sn": "FAC1234701127",
            //				"alias": "1¥1",
            //				"verify": 15,
            //				"stateno": "5",
            //				"state": ""

            //ѯ˴򿨻ݵid
            HashSet<String> attIds = new HashSet();
            for (int j = 0; j < jsonArray.size(); j++) {
                JSONObject resultData = jsonArray.getJSONObject(j);
                attIds.add(resultData.get("id").toString());
            }

            //ͬ
            HashSet<String> existedPunchCardRecordCol = getExistedPunchCardRecordCol(ctx, attIds);
            for (int j = 0; j < jsonArray.size(); j++) {
                JSONObject resultData = jsonArray.getJSONObject(j);
                String attId = resultData.getString("id");
                if (existedPunchCardRecordCol.contains(attId)) {
                    continue;
                }
                ++success;
                String personNumber = resultData.getString("pin");//Ա
                String checktime = resultData.getString("checktime"); //ʱ
                String RecDate = checktime.substring(0, 10);// 
                String address = resultData.getString("alias"); //򿨵ص
                String EquNo = resultData.getString("sn"); //ڻ
                SimpleDateFormat sdfymd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Date punchDate = sdfymd.parse(RecDate + " 00:00:00");
//                Date punchDate = HRTimeWebUtils.stringToShortDate(RecDate, true);
                Timestamp punchCardTime = HRTimeWebUtils.stringToTimestamp(checktime, true);
                // 
                if (punchDate != null && punchCardTime != null) {
                    PunchCardRecordInfo cardInfo = new PunchCardRecordInfo();
                    cardInfo.setNumber(attId);
                    //ڻ
                    cardInfo.setEquipmentNum(EquNo);
                    //Ա
                    cardInfo.setAttendanceNum(personNumber);
                    //
                    cardInfo.setPunchCardDate(punchDate);
                    //ʱ
                    cardInfo.setPunchCardTime(punchCardTime);
                    // Դ
                    cardInfo.setPunchCardSource(PunchCardSourceEnum.attenceMachine);
                    // λ
                    cardInfo.setPunchCardPlace(address);
                    // Ч
                    PunchCardStateEnum punchCardStateEnum;
                    //if ((boolean) resultData.get("isActive")) {
                    //    punchCardStateEnum = PunchCardStateEnum.normal;
                    //} else {
                    //    punchCardStateEnum = PunchCardStateEnum.cancelled;
                    //}
                    punchCardStateEnum = PunchCardStateEnum.normal;
                    cardInfo.setPunchCardState(punchCardStateEnum);
                    ////
                    //cardInfo.setSimpleName(exceptionType);
                    //
                    cardInfo.setDescription("ڻͬ");
                    if (!StringUtils.isEmpty(personNumber)) {
                        //shrϵͳڵԱ;
                        if (!persNoSet.contains(personNumber)) {
                            logger.error("ͬ:personNumber not exist in shr system, personNumber=" + personNumber);
                            continue;
                        }
                        setCardId.add(personNumber);
                        cardColl.add(cardInfo);
                    }
                }
            }
            syncAttendanceRecords(setCardId, cardColl, ctx);
        } catch (Exception e) {
            logger.error("Ž򿨼¼ʧ: " + e.getMessage(), e);
        }
            // ڵ

    }





    private void syncAttendanceRecords(Set<String> setCardId, CoreBaseCollection  cardColl, Context ctx) throws BOSException, EASBizException {
        // ȡڵ
        Map<String, AttendanceFileInfo> attendanceFileMap = AttendanceFileFactory.getLocalInstance(ctx).getPersonByAttendanceNum(setCardId);
        logger.error("attendanceFileMap-----" + attendanceFileMap);
        logger.error("setCardId-----" + setCardId);

        CoreBaseCollection cardCollFinally = new CoreBaseCollection();
        Collection<Map> seccussArray = new ArrayList<>();
        Collection<JSONObject> notExistRecords = new ArrayList<>();
        int success = 0;

        for (int i = 0; i < cardColl.size(); ++i) {
            Map<String, String> map = new HashMap<>();
            // ԭʼ򿨼¼ʵ
            PunchCardRecordInfo cardInfo = (PunchCardRecordInfo) cardColl.get(i);
            JSONObject jsObject = new JSONObject();
            jsObject.put("deviceid", cardInfo.getEquipmentNum()); // deviceid --EquNo
            jsObject.put("userid", cardInfo.getAttendanceNum()); // userid --CardId
            jsObject.put("RecDate", HRTimeWebUtils.dateShortToString(cardInfo.getPunchCardDate())); //checkin_time --RecDate
            jsObject.put("RecTime", HRTimeWebUtils.timestampToString(cardInfo.getPunchCardTime()).substring(11)); //checkin_time --RecTime
            map.put("userid", cardInfo.getAttendanceNum());
            map.put("RecDate", HRTimeWebUtils.dateShortToString(cardInfo.getPunchCardDate()));
            map.put("RecTime", HRTimeWebUtils.timestampToString(cardInfo.getPunchCardTime()).substring(11));

            // ԱǷпڵ
            if (attendanceFileMap.containsKey(cardInfo.getAttendanceNum())) {
                ++success;
                // λ ڻ ĵַ
                cardInfo.setPunchCardPlace(cardInfo.getPunchCardPlace());
                // Դ
                cardInfo.setPunchCardSource(PunchCardSourceEnum.attenceMachine);
                // ʵ
                AttendanceFileInfo attendanceFileInfo = attendanceFileMap.get(cardInfo.getAttendanceNum());
                // 
                cardInfo.setProposer(attendanceFileInfo.getProposer());
                // HR ֯
                cardInfo.setHrOrgUnit(attendanceFileInfo.getHrOrgUnit());
                // ֯
                cardInfo.setAdminOrgUnit(attendanceFileInfo.getAdminOrgUnit());
                cardInfo.setCreator((UserInfo) ctx.get("UserInfo"));
                cardInfo.setLastUpdateUser((UserInfo) ctx.get("UserInfo"));
                cardInfo.setCreateTime(new Timestamp(new Date().getTime()));
                cardInfo.setLastUpdateTime(new Timestamp(new Date().getTime()));
                cardCollFinally.add(cardInfo);
                seccussArray.add(map);
            } else {
                notExistRecords.add(jsObject);
            }
        }

        logger.error("ͬ浽hr-----" + cardCollFinally.size());
        logger.error("ͬд-----" + success);
        logger.error("ͬʱڿڵ---" + notExistRecords);
        logger.error("ͬʱڿڵ-----" + notExistRecords.size());
        logger.error("seccussArray--Size-----" + seccussArray.size());
        logger.error("seccussArray-----" + seccussArray);

        if (cardCollFinally.size() > 0) {
            IObjectPK[] iObjectPKS = PunchCardRecordFactory.getLocalInstance(ctx).saveBatchData(cardCollFinally);
            logger.error("iObjectPKS----" + iObjectPKS.length);
        }
    }










    private HashSet<String>  getExistedPunchCardRecordCol(Context ctx, HashSet<String> attId) throws BOSException {
    	HashSet<String> hashSet = new HashSet<String>();
        EntityViewInfo evi = new EntityViewInfo();
        FilterInfo fi = new FilterInfo();
        evi.setFilter(fi);
        evi.getSelector().add(new SelectorItemInfo("number"));
        fi.getFilterItems().add(new FilterItemInfo("number", attId, CompareType.INCLUDE));
        PunchCardRecordCollection existColl = PunchCardRecordFactory.getLocalInstance(ctx).getPunchCardRecordCollection(evi);
        for(int i =0 ;i<existColl.size();i++) {
        	PunchCardRecordInfo punchCardRecordInfo = existColl.get(i);
        	hashSet.add(punchCardRecordInfo.getNumber());
        }
        return hashSet;

    }



    
}