package com.kingdee.eas.custom.dormitorysystem.application.service;

import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.bsf.service.app.IHRMsfService;
import com.kingdee.bos.dao.ormapping.ObjectUuidPK;
import com.kingdee.bos.metadata.entity.*;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.basedata.person.Genders;
import com.kingdee.eas.basedata.person.IPerson;
import com.kingdee.eas.basedata.person.PersonFactory;
import com.kingdee.eas.basedata.person.PersonInfo;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.dormitorysystem.application.CheckInApplicationEntryCollection;
import com.kingdee.eas.custom.dormitorysystem.application.CheckInApplicationEntryFactory;
import com.kingdee.eas.custom.dormitorysystem.application.CheckInApplicationEntryInfo;
import com.kingdee.eas.custom.dormitorysystem.application.ICheckInApplicationEntry;
import com.kingdee.eas.custom.dormitorysystem.dormitory.DormitoryCollection;
import com.kingdee.eas.custom.dormitorysystem.dormitory.DormitoryFactory;
import com.kingdee.eas.custom.dormitorysystem.dormitory.DormitoryInfo;
import com.kingdee.eas.custom.dormitorysystem.dormitory.IDormitory;
import com.kingdee.eas.custom.dormitorysystem.occupants.IOccupants;
import com.kingdee.eas.custom.dormitorysystem.occupants.OccupantsCollection;
import com.kingdee.eas.custom.dormitorysystem.occupants.OccupantsFactory;
import com.kingdee.eas.custom.dormitorysystem.occupants.OccupantsInfo;
import com.kingdee.eas.custom.dormitorysystem.otherbasicdata.PersonDormitoryTypeInfo;
import com.kingdee.eas.hr.ats.AtsUtil;
import com.kingdee.shr.base.syssetting.exception.ShrWebBizException;
import com.kingdee.util.DateTimeUtils;
import com.kingdee.util.StringUtils;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @Description סںڻȡסid
 * @Date 2025/4/27 16:06
 * @Created by Heyuan
 */
public class GetOptionalRoomIdsService implements IHRMsfService {
    private static Logger logger = Logger.getLogger(GetOptionalRoomIdsService.class);


    @Override
    public Object process(
            Context ctx,
            Map<String, Object> params
    ) throws EASBizException, BOSException {
        StringBuilder errorMsg = new StringBuilder();
        Set<String> dormitoryIdSet = new HashSet<String>();
        try {
            //ס
            String checkInDateStr = (String) params.get("checkInDate");
            if (StringUtils.isEmpty(checkInDateStr)) {
                throw new BOSException("סڲΪ!");
            }
            //
            String checkoutDateStr = (String) params.get("checkoutDate");
            if (StringUtils.isEmpty(checkInDateStr)) {
                throw new BOSException("ڲΪ!");
            }
            //Աid
            String personId = (String) params.get("personId");
            if (StringUtils.isEmpty(personId)) {
                throw new BOSException("ԱidΪ!");
            }
            //ס
            Date checkInDate = DateTimeUtils.parseDate(checkInDateStr, "yyyy-MM-dd");
            Date checkoutDate = DateTimeUtils.parseDate(checkoutDateStr, "yyyy-MM-dd");
            //ס,Ա,Աѯѡ
            IPerson iPerson = PersonFactory.getLocalInstance(ctx);
            SelectorItemCollection sic = new SelectorItemCollection();
            sic.add("gender");
            sic.add("personType.*");
            sic.add("name");
            PersonInfo personInfo = iPerson.getPersonInfo(new ObjectUuidPK(personId), sic);
            Genders gender = personInfo.getGender();
            //Ա
            PersonDormitoryTypeInfo personType = (PersonDormitoryTypeInfo) personInfo.get("personType");
            if (personType == null) {
                throw new ShrWebBizException("Ա[" + personInfo.getName() + "]ԱΪգ뼰ʱά");
            }
            String personTypeId = personType.getId().toString();
            Map<String, Map<String, Object>> dormitoryMap = new HashMap<>();
            //1.ԱѯпסϢ
            List<DormitoryInfo> dormitoryInfoList = getDormitoryInfos(ctx, gender, dormitoryIdSet);
            //2.סںסϢÿס
            getCheckInInfoByOccupants(ctx, checkInDate, checkoutDate, dormitoryMap, dormitoryIdSet);
            //3.סںס뵥״̬Ϊеס
            getCheckInInfoByCheckInApplication(ctx, checkInDate, checkoutDate, dormitoryMap, dormitoryIdSet);
            //4.ʣס: ס - ס
            for (int i = 0; i < dormitoryInfoList.size(); i++) {
                DormitoryInfo dormitoryInfo = dormitoryInfoList.get(i);
                String dormitoryId = dormitoryInfo.getId().toString();
                Map<String, Object> dormCheckInMap = dormitoryMap.get(dormitoryId);
                if (dormCheckInMap != null) {
                    String personDormitoryTypeId = (String) dormCheckInMap.getOrDefault("personDormitoryTypeId", "");
                    //жǷ͸ԱǷͬ
                    if (personDormitoryTypeId.equals(personTypeId)) {
                        //ס
                        int dormCheckInCount = (int) dormCheckInMap.get("count");
                        //ס
                        int occupancyCapacity = dormitoryInfo.getOccupancyCapacity();
                        if (occupancyCapacity - dormCheckInCount <= 0) {
                            dormitoryIdSet.remove(dormitoryId);
                        }
                    } else {
                        dormitoryIdSet.remove(dormitoryId);
                    }
                }
            }
            return AtsUtil.convertSetToString(dormitoryIdSet);
        } catch (Exception e) {
            logger.error(e);
            errorMsg.append(e.getMessage()).append(System.lineSeparator());
        }
        Map result = new HashMap();
        if (errorMsg.length() > 0) {
            result.put("code", 500);
            result.put("errorMsg", errorMsg.toString());
        } else {
            result.put("code", 200);
            result.put("dormitoryIds", AtsUtil.convertSetToString(dormitoryIdSet));

        }
        return result;
    }

    /**
     * סںס뵥״̬Ϊеס
     *
     * @param ctx
     * @param checkInDate  ס
     * @param checkoutDate 
     * @param dormitoryMap
     * @throws BOSException
     */
    private void getCheckInInfoByCheckInApplication(
            Context ctx,
            Date checkInDate,
            Date checkoutDate,
            Map<String, Map<String, Object>> dormitoryMap,
            Set<String> dormitoryIdSet) throws BOSException, ShrWebBizException {
        ICheckInApplicationEntry iCheckInApplicationEntry = CheckInApplicationEntryFactory.getLocalInstance(ctx);
        FilterInfo filterInfo = new FilterInfo();
        FilterItemCollection filterItems = filterInfo.getFilterItems();
        //
        filterItems.add(new FilterItemInfo("checkInRoom", dormitoryIdSet, CompareType.INCLUDE));
        //ססں֮
        filterItems.add(new FilterItemInfo("checkInDate", checkInDate, CompareType.LESS_EQUALS));
        filterItems.add(new FilterItemInfo("checkoutDate", checkInDate, CompareType.GREATER_EQUALS));
        //ססں֮
        filterItems.add(new FilterItemInfo("checkInDate", checkInDate, CompareType.GREATER_EQUALS));
        filterItems.add(new FilterItemInfo("checkInDate", checkoutDate, CompareType.LESS_EQUALS));
        //״̬
        //filterItems.add(new FilterItemInfo("parent.billState", "3"));
        filterInfo.setMaskString("#0 and ((#1 and #2) or (#3 and #4)) ");
        //filterInfo.setMaskString("#0 and ((#1 and #2) or (#3 and #4)) and #5 ");
        SelectorItemCollection sic = new SelectorItemCollection();
        sic.add("checkInRoom.id");
        sic.add("person.personType.id");
        EntityViewInfo viewInfo = EntityViewInfo.getInstance(filterInfo, sic, null);
        CheckInApplicationEntryCollection checkInApplicationEntryCol = iCheckInApplicationEntry.getCheckInApplicationEntryCollection(viewInfo);
        for (int i = 0; i < checkInApplicationEntryCol.size(); i++) {
            CheckInApplicationEntryInfo checkInApplicationEntryInfo = checkInApplicationEntryCol.get(i);
            //
            DormitoryInfo checkInRoom = checkInApplicationEntryInfo.getCheckInRoom();
            String dormitoryId = checkInRoom.getId().toString();
            PersonInfo person = checkInApplicationEntryInfo.getPerson();
            String personName = person.getName();
            //Ա
            PersonDormitoryTypeInfo personDormitoryType = (PersonDormitoryTypeInfo) person.get("personType");
            if (personDormitoryType == null) {
                throw new ShrWebBizException("Ա[" + personName + "]ԱΪգ뼰ʱά");
            }
            if (dormitoryMap.containsKey(dormitoryId)) {
                Map<String, Object> map = dormitoryMap.get(dormitoryId);
                //ۼס
                int count = (int) map.getOrDefault("count", 0);
                map.put("count", count + 1);
            } else {
                Map<String, Object> map = new HashMap<>();
                map.put("count", 1);
                map.put("personDormitoryTypeId", personDormitoryType.getId().toString());
                dormitoryMap.put(dormitoryId, map);
            }
        }
    }

    /**
     * סںסϢÿס
     *
     * @param ctx
     * @param checkInDate    ס
     * @param checkoutDate   
     * @param dormitoryIdSet
     * @return
     * @throws BOSException
     */
    protected void getCheckInInfoByOccupants(
            Context ctx,
            Date checkInDate,
            Date checkoutDate,
            Map<String, Map<String, Object>> dormitoryMap,
            Set<String> dormitoryIdSet
    ) throws BOSException {
        IOccupants iOccupants = OccupantsFactory.getLocalInstance(ctx);
        FilterInfo filterInfo = new FilterInfo();
        FilterItemCollection filterItems = filterInfo.getFilterItems();
        //
        filterItems.add(new FilterItemInfo("Dormitory", dormitoryIdSet, CompareType.INCLUDE));
        //ססں֮
        filterItems.add(new FilterItemInfo("checkInDate", checkInDate, CompareType.LESS_EQUALS));
        filterItems.add(new FilterItemInfo("checkoutDate", checkInDate, CompareType.GREATER_EQUALS));
        //ססں֮
        filterItems.add(new FilterItemInfo("checkInDate", checkInDate, CompareType.GREATER_EQUALS));
        filterItems.add(new FilterItemInfo("checkInDate", checkoutDate, CompareType.LESS_EQUALS));
        filterInfo.setMaskString("#0 and ((#1 and #2) or (#3 and #4)) ");
        SelectorItemCollection sic = new SelectorItemCollection();
        sic.add("Dormitory.id");
        sic.add("PersonDormitoryType.id");
        EntityViewInfo viewInfo = EntityViewInfo.getInstance(filterInfo, sic, null);
        OccupantsCollection occupantsCollection = iOccupants.getOccupantsCollection(viewInfo);
        for (int i = 0; i < occupantsCollection.size(); i++) {
            OccupantsInfo occupantsInfo = occupantsCollection.get(i);
            //
            DormitoryInfo dormitory = occupantsInfo.getDormitory();
            String dormitoryId = dormitory.getId().toString();
            //Ա
            PersonDormitoryTypeInfo personDormitoryType = occupantsInfo.getPersonDormitoryType();
            if (dormitoryMap.containsKey(dormitoryId)) {
                Map<String, Object> map = dormitoryMap.get(dormitoryId);
                //ۼס
                int count = (int) map.getOrDefault("count", 0);
                map.put("count", count + 1);
            } else {
                Map<String, Object> map = new HashMap<>();
                map.put("count", 1);
                map.put("personDormitoryTypeId", personDormitoryType.getId().toString());
                dormitoryMap.put(dormitoryId, map);
            }
        }
    }

    /**
     * ѯп÷Ϣ
     *
     * @param ctx
     * @param gender Ա
     * @return
     * @throws BOSException
     */
    protected List<DormitoryInfo> getDormitoryInfos(Context ctx, Genders gender, Set<String> dormitoryIdSet) throws BOSException {
        IDormitory iDormitory = DormitoryFactory.getLocalInstance(ctx);
        FilterInfo filterInfo = new FilterInfo();
        FilterItemCollection filterItems = filterInfo.getFilterItems();
        //Ϊ
        filterItems.add(new FilterItemInfo("RoomClassification", 3));
        //.Ա
        filterItems.add(new FilterItemInfo("DormitoryType.gender", gender.getValue()));
        filterItems.add(new FilterItemInfo("DormitoryType.gender", null));
        //״̬Ϊ
        filterItems.add(new FilterItemInfo("state", "1"));
//        SelectorItemCollection sic = new SelectorItemCollection();
//        sic.add("Dormitory.id");
//        sic.add("PersonDormitoryType.id");
        filterInfo.setMaskString("#0 and (#1 or #2) and #3");
        EntityViewInfo viewInfo = EntityViewInfo.getInstance(filterInfo, null, null);
        //1.ѯϢ
        List<DormitoryInfo> dormitoryInfoList = new ArrayList<DormitoryInfo>();
        DormitoryCollection dormitoryCollection = iDormitory.getDormitoryCollection(viewInfo);
        for (int i = 0; i < dormitoryCollection.size(); i++) {
            DormitoryInfo dormitoryInfo = dormitoryCollection.get(i);
            dormitoryInfoList.add(dormitoryInfo);
            dormitoryIdSet.add(dormitoryInfo.getId().toString());
        }
        return dormitoryInfoList;
    }
}
