package com.kingdee.eas.custom.dormitorysystem.waterpower.utils;

import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.metadata.entity.EntityViewInfo;
import com.kingdee.bos.metadata.entity.FilterInfo;
import com.kingdee.bos.metadata.entity.FilterItemCollection;
import com.kingdee.bos.metadata.entity.FilterItemInfo;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.custom.dormitorysystem.dormitory.DormitoryFactory;
import com.kingdee.eas.custom.dormitorysystem.dormitory.DormitoryInfo;
import com.kingdee.eas.custom.dormitorysystem.dormitory.IDormitory;
import com.kingdee.eas.custom.dormitorysystem.waterpower.IWaterPowerMeterReading;
import com.kingdee.eas.custom.dormitorysystem.waterpower.WaterPowerMeterReadingCollection;
import com.kingdee.eas.custom.dormitorysystem.waterpower.WaterPowerMeterReadingFactory;
import com.kingdee.eas.custom.dormitorysystem.waterpower.WaterPowerMeterReadingInfo;
import com.kingdee.shr.base.syssetting.exception.ShrWebBizException;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qingwu
 * @date 2025/4/22
 * @apiNote
 */
public class WaterPowerUtil {
    static SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM");

    public static void verifyDatEexists(Context ctx, Date years, String dormNumber, String id) throws ShrWebBizException {
        try {
            FilterInfo filterInfo = new FilterInfo();
            FilterItemCollection filterItems = filterInfo.getFilterItems();
            filterItems.add(new FilterItemInfo("years", years));
            filterItems.add(new FilterItemInfo("dormitory.number", dormNumber));
            if (id != "") {
                filterItems.add(new FilterItemInfo("id", id, CompareType.NOTEQUALS));
            }
            IDormitory iDormitory = DormitoryFactory.getLocalInstance(ctx);
            DormitoryInfo dormitoryInfo = iDormitory.getDormitoryInfo("where number = '" + dormNumber + "'");
            IWaterPowerMeterReading iWaterPowerMeterReading = WaterPowerMeterReadingFactory.getLocalInstance(ctx);
            boolean exists = iWaterPowerMeterReading.exists(filterInfo);
            if (exists) {
                throw new ShrWebBizException(dormitoryInfo.getName() + "Ѿ" + formatter.format(years) + "µĳϢ");
            }
        } catch (
                Exception e) {
            e.printStackTrace();
            throw new ShrWebBizException(e);
        }

    }

    public static Map getThisMonthWaterPowerTons(Context ctx, String years, String dormNumber, BigDecimal totalWaterTons, BigDecimal totalElecDegrees) throws BOSException {
        Map<String, String> dateMap = getDate(years);
        String previousMonth = dateMap.get("previousMonth");
        String twoMonthsAgo = dateMap.get("twoMonthsAgo");
        //ȡϸµˮܶ
        //ˮܶ
        BigDecimal water = BigDecimal.ZERO;
        //ܶ
        BigDecimal power = BigDecimal.ZERO;
        //ˮ ?ʵˮ㣨-£-£绹ֱʾˮܶ
        BigDecimal curMonthWater = BigDecimal.ZERO;
        //µ ?ʵõ㣨-£-£绹ֱʾõܶ
        BigDecimal curMonthElecDegrees = BigDecimal.ZERO;
        Map<String, BigDecimal> previousMonthMap = getWaterPower(ctx, previousMonth, dormNumber);
        if (previousMonthMap.size() > 0) {
            water = previousMonthMap.get("water");
            power = previousMonthMap.get("power");
            curMonthWater = totalWaterTons.subtract(water);
            curMonthElecDegrees = totalElecDegrees.subtract(power);
        } else {
            //ȡϸµˮܶ
            previousMonthMap = getWaterPower(ctx, twoMonthsAgo, dormNumber);
            if (previousMonthMap.size() > 0) {
                water = previousMonthMap.get("water");
                power = previousMonthMap.get("power");
                curMonthWater = totalWaterTons.subtract(water);
                curMonthElecDegrees = totalElecDegrees.subtract(power);
            }
            curMonthWater = totalWaterTons;
            curMonthElecDegrees = totalElecDegrees;
        }
        Map result = new HashMap();
        result.put("curMonthWater", curMonthWater.compareTo(BigDecimal.ZERO) <= 0 ? BigDecimal.ZERO : curMonthWater);//ˮ
        result.put("curMonthElecDegrees", curMonthElecDegrees.compareTo(BigDecimal.ZERO) <= 0 ? BigDecimal.ZERO : curMonthElecDegrees);//µ
        return result;
    }

    /**
     * ȡˮ
     *
     * @param date
     * @param dormNumber
     * @return
     * @throws BOSException
     */
    public static Map getWaterPower(Context ctx, String date, String dormNumber) throws BOSException {
        Map<String, BigDecimal> map = new HashMap();
        FilterInfo filterInfo = new FilterInfo();
        FilterItemCollection filterItems = filterInfo.getFilterItems();
        //ϸ:previousMonth   ϸ:
        filterItems.add(new FilterItemInfo("years", date + "-01 00:00:00"));
        filterItems.add(new FilterItemInfo("dormitory.number", dormNumber));
        EntityViewInfo entityViewInfo = EntityViewInfo.getInstance(filterInfo, null, null);
        //ˮÿ³Ϣ
        IWaterPowerMeterReading iWaterPowerMeterReading = WaterPowerMeterReadingFactory.getLocalInstance(ctx);
        WaterPowerMeterReadingCollection waterPowerMeterReadingCollection = iWaterPowerMeterReading.getWaterPowerMeterReadingCollection(entityViewInfo);

        for (int i = 0; i < waterPowerMeterReadingCollection.size(); i++) {
            WaterPowerMeterReadingInfo waterPowerMeterReadingInfo = waterPowerMeterReadingCollection.get(i);
            map.put("water", waterPowerMeterReadingInfo.getTotalWaterTons());
            map.put("power", waterPowerMeterReadingInfo.getTotalElecDegrees());
        }
        return map;
    }

    public static Map getDate(String dateStr) {
        Map map = new HashMap();
        // ڸʽ
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
        // ַתΪ YearMonth 
        YearMonth targetDate = YearMonth.parse(dateStr, formatter);
        // ȡϸ
        YearMonth previousMonth = targetDate.minusMonths(1);
        map.put("previousMonth", formatter.format(previousMonth));
        // ȡϸ
        YearMonth twoMonthsAgo = targetDate.minusMonths(2);
        map.put("twoMonthsAgo", formatter.format(twoMonthsAgo));
        return map;

    }
}
